@@ -28,7 +28,8 @@
 
 #include <glib-object.h>
 
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4util/libxfce4util.h>
+#include <libxfce4ui/libxfce4ui.h>
 #include <libwnck/libwnck.h>
 
 #include "xfdesktop-icon-view.h"
@@ -37,6 +38,8 @@
 #include "xfce-desktop.h"
 
 
+static gboolean __wnck_has_new_active_workspace_changed_signal = TRUE;
+
 static void xfdesktop_window_icon_manager_set_property(GObject *object,
                                                        guint property_id,
                                                        const GValue *value,
@@ -90,6 +93,7 @@ static void
 xfdesktop_window_icon_manager_class_init(XfdesktopWindowIconManagerClass *klass)
 {
     GObjectClass *gobject_class = (GObjectClass *)klass;
+    guint sig_id;
     
     g_type_class_add_private(klass, sizeof(XfdesktopWindowIconManagerPrivate));
     
@@ -103,6 +107,21 @@ xfdesktop_window_icon_manager_class_init(XfdesktopWindowIconManagerClass *klass)
                                                         GDK_TYPE_SCREEN,
                                                         G_PARAM_READWRITE
                                                         | G_PARAM_CONSTRUCT_ONLY));
+
+    /* make sure the class is alive; g_signal_lookup() fails otherwise */
+    if(!g_type_class_peek(WNCK_TYPE_SCREEN))
+        g_type_class_ref(WNCK_TYPE_SCREEN);
+    sig_id = g_signal_lookup("active-workspace-changed", WNCK_TYPE_SCREEN);
+    if(sig_id) {
+        GSignalQuery query;
+
+        g_signal_query(sig_id, &query);
+        if(query.n_params == 0)
+            __wnck_has_new_active_workspace_changed_signal = FALSE;
+        else
+            __wnck_has_new_active_workspace_changed_signal = TRUE;
+        DBG("active-workspace-changed signal has %d param(s)\n", query.n_params);
+    }
 }
 
 static void
@@ -172,8 +191,8 @@ xfdesktop_window_icon_manager_icon_view_manager_init(XfdesktopIconViewManagerIfa
 
 
 static void
-xfdesktop_window_icon_manager_icon_selected_cb(XfdesktopIconView *icon_view,
-                                               gpointer user_data)
+xfdesktop_window_icon_manager_icon_selection_changed_cb(XfdesktopIconView *icon_view,
+                                                        gpointer user_data)
 {
     XfdesktopWindowIconManager *wmanager = user_data;
     GList *selected;
@@ -184,7 +203,8 @@ xfdesktop_window_icon_manager_icon_selected_cb(XfdesktopIconView *icon_view,
         gint ws = xfdesktop_window_icon_get_workspace(window_icon);
         wmanager->priv->icon_workspaces[ws]->selected_icon = window_icon;
         g_list_free(selected);
-    }
+    } else
+        wmanager->priv->icon_workspaces[wmanager->priv->active_ws_num]->selected_icon = NULL;
 }
 
 static XfdesktopWindowIcon *
@@ -216,13 +236,21 @@ xfdesktop_add_window_icons_foreach(gpointer key,
 
 static void
 workspace_changed_cb(WnckScreen *wnck_screen,
-                     WnckWorkspace *previous_workspace,
+                     gpointer arg1,
                      gpointer user_data)
 {
-    XfdesktopWindowIconManager *wmanager = XFDESKTOP_WINDOW_ICON_MANAGER(user_data);
+    XfdesktopWindowIconManager *wmanager;
     gint n;
     WnckWorkspace *ws;
-    
+
+    /* as of libwnck 2.20, the "active-workspace-changed" signal has
+     * a new prototype with an extra argument.  previous versions don't
+     * have this argument, so we fudge this here (bug 4395). */
+    if(G_LIKELY(__wnck_has_new_active_workspace_changed_signal))
+        wmanager = XFDESKTOP_WINDOW_ICON_MANAGER(user_data);
+    else
+        wmanager = XFDESKTOP_WINDOW_ICON_MANAGER(arg1);
+
     ws = wnck_screen_get_active_workspace(wmanager->priv->wnck_screen);
     if(!WNCK_IS_WORKSPACE(ws)) {
         DBG("got weird failure of wnck_screen_get_active_workspace(), bailing");
@@ -362,7 +390,8 @@ window_state_changed_cb(WnckWindow *window,
         i = 0;
         max_i = wmanager->priv->nworkspaces;
     } else {
-        g_return_if_fail(ws_num != -1);
+        if(ws_num == -1)
+            return;
         i = ws_num;
         max_i = i + 1;
     }
@@ -531,8 +560,8 @@ xfdesktop_window_icon_manager_real_init(XfdesktopIconViewManager *manager,
     
     wmanager->priv->icon_view = icon_view;
     xfdesktop_icon_view_set_selection_mode(icon_view, GTK_SELECTION_SINGLE);
-    g_signal_connect(G_OBJECT(icon_view), "icon-selected",
-                     G_CALLBACK(xfdesktop_window_icon_manager_icon_selected_cb),
+    g_signal_connect(G_OBJECT(icon_view), "icon-selection-changed",
+                     G_CALLBACK(xfdesktop_window_icon_manager_icon_selection_changed_cb),
                      wmanager);
     
     wmanager->priv->desktop = gtk_widget_get_toplevel(GTK_WIDGET(icon_view));
@@ -620,7 +649,7 @@ xfdesktop_window_icon_manager_fini(XfdesktopIconViewManager *manager)
     
     xfdesktop_icon_view_remove_all(wmanager->priv->icon_view);
     g_signal_handlers_disconnect_by_func(G_OBJECT(wmanager->priv->icon_view),
-                                         G_CALLBACK(xfdesktop_window_icon_manager_icon_selected_cb),
+                                         G_CALLBACK(xfdesktop_window_icon_manager_icon_selection_changed_cb),
                                          wmanager);
     
     for(i = 0; i < wmanager->priv->nworkspaces; ++i) {
